/***************************************************************************
 *   Copyright (C) 2005 by Emiliano Gulmini   *
 *   emi_barbarossa@yahoo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//QT Headers
#include <qdom.h>
#include <qmap.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>

//KDE Headers
#include <kmessagebox.h>

//Local Headers
#include "kalcoolusrc.h"
#include "functionsdb.h"

using namespace Global;

FunctionsDB::FunctionsDB(const QString& dbPath)
{
  m_functionsDatabasePath = dbPath;
}

FunctionsDB::~FunctionsDB()
{
}

/*!
    \fn FunctionsDB::readFromDatabase()
 */
LanguagesMap FunctionsDB::readFromDatabase()
{
  QDomDocument dbDocument;
  QFile functionsDataBaseFile(m_functionsDatabasePath);
  if(not functionsDataBaseFile.open(IO_ReadOnly))
     return LanguagesMap();
  if(not dbDocument.setContent(&functionsDataBaseFile))
    {
      functionsDataBaseFile.close();
      return LanguagesMap();
    }
  functionsDataBaseFile.close();

  LanguagesMap l;
  l["python"] = getPythonCodeFromDatabase(dbDocument);
  l["bc"] = getBCCodeFromDatabase(dbDocument);

  return l;
}

/*!
    \fn FunctionsDB::getBCCodeFromDatabase(const QDomDocument& db)
 */
FunctionsCodeMap FunctionsDB::getBCCodeFromDatabase(const QDomDocument& db)
{
  // print out the element names of all elements that are direct children
  // of the outermost element.
  QDomElement docElem = db.documentElement();

  QDomNode n = docElem.firstChild();
  FunctionsCodeMap fcMap;
  while(not n.isNull())
    {
      QDomElement e = n.toElement(); // try to convert the node to an element.
      if(not e.isNull())
        {
          if(e.attribute("language") == "bc")
            {
              QString userFunctionName = e.attribute("name"),
                      userFunctionArguments = e.attribute("arguments"),
                      userFunctionBody = e.text();
                      fcMap[userFunctionName] = QString("define %1 (%2) {\n%3\n}\n").arg(userFunctionName).
                                   arg(userFunctionArguments).
                                   arg(userFunctionBody);
            }
        }
      n = n.nextSibling();
    }
  return fcMap;
}

/*!
    \fn FunctionsDB::getPythonCodeFromDatabase(const QDomDocument& db)
 */
FunctionsCodeMap FunctionsDB::getPythonCodeFromDatabase(const QDomDocument& db)
{
  // print out the element names of all elements that are direct children
  // of the outermost element.
  QDomElement docElem = db.documentElement();

  QDomNode n = docElem.firstChild();
  FunctionsCodeMap fcMap;
  while(not n.isNull())
    {
      QDomElement e = n.toElement(); // try to convert the node to an element.
      if(not e.isNull())
        {
          if(e.attribute("language") == "python")
            {
              QString userFunctionName = e.attribute("name"),
                      userFunctionArguments = e.attribute("arguments"),
                      userFunctionBody = e.text();
                      fcMap[userFunctionName] = QString("def %1 (%2):\n%3\n\n").arg(userFunctionName).
                               arg(userFunctionArguments).
                               arg(userFunctionBody);
            }
        }
      n = n.nextSibling();
    }
  return fcMap;
}

/*!
    \fn FunctionsDB::writeToDatabase(const QMap<QString, QString>& m)
 */
void FunctionsDB::writeToDatabase(const LanguagesMap& m)
{
  m_languagesMap = m;
  /** Opens an xml file to use as database */
  QDomDocument dbDocument;
  dbDocument.appendChild(dbDocument.createElement("functions"));
  /** adds to database BC and Python user defined functions */
  putBCIntoDatabase(m,dbDocument);
  putPythonIntoDatabase(m,dbDocument);
  /** writes database on disk */
  QFile functionsDataBaseFile(m_functionsDatabasePath);
  if(not functionsDataBaseFile.open(IO_WriteOnly))
    KMessageBox::error(0,"Cannot store functions into database");
  else
    {
      QTextStream functionsDataBaseStream(&functionsDataBaseFile);
      functionsDataBaseStream << "<?xml version=\"1.0\" ?>\n";
      functionsDataBaseStream << dbDocument.toString(2);
      functionsDataBaseFile.close();
    }
}

/*!
    \fn FunctionsDB::createBCDatabase(const LanguagesMap& m, QDomDocument& db)
 */
void FunctionsDB::putBCIntoDatabase(const LanguagesMap& m, QDomDocument& db)
{
  // This function is used by FunctionsDB::writeToDatabase
  FunctionsCodeMap::ConstIterator it;
  for(it = m["bc"].constBegin(); it != m["bc"].constEnd(); ++it)
    {
      QString functionCode = (*it).data();
      QRegExp r = QRegExp(bcFunctionNamePattern);
              r.search(functionCode);
      QString userFunctionName = r.cap(1);
              r = QRegExp(bcFunctionArgumentsPattern);
              r.search(functionCode);
      QString userFunctionArguments = r.cap(1);
              r = QRegExp(bcFunctionBodyPattern);
              r.search(functionCode);
      QString userFunctionBody = r.cap(1);

      QDomElement tempElement = db.createElement("define");
      tempElement.setAttribute("name",userFunctionName);
      tempElement.setAttribute("arguments",userFunctionArguments);
      tempElement.setAttribute("language","bc");
      QDomCDATASection cdata = db.createCDATASection(userFunctionBody);
      tempElement.appendChild(cdata);
      db.firstChild().appendChild(tempElement);
    }
}

/*!
    \fn FunctionsDB::createPythonDatabase(const LanguagesMap& m, QDomDocument& db)
 */
void FunctionsDB::putPythonIntoDatabase(const LanguagesMap& m, QDomDocument& db)
{
  // This function is used by FunctionsDB::writeToDatabase
  FunctionsCodeMap::ConstIterator it;
  for(it = m["python"].begin(); it != m["python"].end(); ++it)
    {
      QString functionCode = (*it).data();
      QRegExp r = QRegExp(pythonFunctionNamePattern);
              r.search(functionCode);
      QString userFunctionName = r.cap(1);
              r = QRegExp(pythonFunctionArgumentsPattern);
              r.search(functionCode);
      QString userFunctionArguments = r.cap(1);
              r = QRegExp(pythonFunctionBodyPattern);
              r.search(functionCode);
      QString userFunctionBody = r.cap(1);

      QDomElement tempElement = db.createElement("define");
      tempElement.setAttribute("name",userFunctionName);
      tempElement.setAttribute("arguments",userFunctionArguments);
      tempElement.setAttribute("language","python");
      QDomCDATASection cdata = db.createCDATASection(userFunctionBody);
      tempElement.appendChild(cdata);
      db.firstChild().appendChild(tempElement);
    }
}
