/***************************************************************************
 *   Copyright (C) 2005 by Emiliano Gulmini   *
 *   emi_barbarossa@yahoo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//QT Headers
#include <qlabel.h>
#include <qpushbutton.h>
#include <qstringlist.h>
#include <qcstring.h>
#include <qregexp.h>
#include <qtable.h>
#include <qmap.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qspinbox.h>
#include <qwidgetstack.h>
#include <qiconview.h>
#include <qfont.h>
#include <qcolor.h>
#include <qgroupbox.h>
#include <qcheckbox.h>
#include <qbrush.h>
#include <qfontmetrics.h>
#include <qsize.h>

//KDE Headers
#include <klocale.h>
#include <ktextedit.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <klistview.h>
#include <klineedit.h>
#include <kpushbutton.h>
#include <kconfig.h>
#include <kstandarddirs.h>
#include <kcombobox.h>
#include <kfontrequester.h>
#include <kcolorcombo.h>
#include <khelpmenu.h>
#include <kaboutdata.h>
#include <kurlrequester.h>

//Local and Other Headers
#include "kalcoolus.h"
#include "constantsdb.h"
#include "functionsdb.h"
#include "pythoncodegenerator.h"
#include "bccodegenerator.h"
#include "codegenerator.h"
#include "configuration.h"
#include "highlighter.h"
#include "kalcoolusrc.h"
#include "kalcoolusimplementif.h"

using namespace Global;

Kalcoolus::Kalcoolus() : Kbc( 0, "Kalcoolus" )
{
  // Initializing config file 
  m_config = new KConfig("kalcoolusrc");

  // Initializing display 
  m_inputHighlighter = new Highlighter(m_inputDisplay);
  m_resultHighlighter = new Highlighter(m_resultDisplay);
  m_functionEditorHighlighter = new Highlighter(m_editorUserFunctionBody);
  m_inputDisplay->setText("\n");
  m_inputDisplay->setCursorPosition(0,0);

  // Temporary file that contains the code to be execute when we press '=' button
  m_bcScriptFilePath = locateLocal("data", "kalcoolus/program_bc");
  m_pythonScriptFilePath = locateLocal("data", "kalcoolus/program_python.py");

  // Loading constants from database 
  m_constantsDatabasePath = locateLocal("data", "kalcoolus/constantsDB.xml");
  m_constantsDB = new ConstantsDB(m_constantsDatabasePath, m_constantsListview);
  m_constantsDB->readFromDatabase();
  loadConstantsIntoBox();

  // Loading functions from database 
  m_functionsDatabasePath = locateLocal("data","kalcoolus/functionsDB.xml");
  m_functionsDB = new FunctionsDB(m_functionsDatabasePath);
  m_languagesMap = m_functionsDB->readFromDatabase();
  loadFunctionsIntoBoxes();

  // Loading configuration from rc file 
  m_configuration = new Configuration(m_config);
  loadBCSettings();
  loadPythonSettings();
  activateButtons();
  loadFontSettings();
  loadOptionsSettings();

  // Mapping buttons into symbols 
  mapButtons();

  // Connecting signals 
  ConnectKeyboardPage();
  ConnectConstantsPage();
  ConnectConfigurationPage();
  ConnectFunctionsPage();

  // Creating help menu
  createHelp();

  // Initializing DCOP interface
  m_interface = new KalcoolusImplementIf(m_inputDisplay, m_resultDisplay, m_bEqual);
}

Kalcoolus::~Kalcoolus()
{
  if(m_configuration) delete m_configuration;
  if(m_functionsDB) delete m_functionsDB;
  if(m_constantsDB) delete m_constantsDB;
  if(m_config) delete m_config;
  if(m_inputHighlighter) delete m_inputHighlighter;
  if(m_resultHighlighter) delete m_resultHighlighter;
  if(m_functionEditorHighlighter) delete m_functionEditorHighlighter;
  if(m_interface) delete m_interface;
}
/*******************************************************************/
/************************ PRIVATE SLOTS ****************************/
/*******************************************************************/

// Keyboard Page Slots 
/*!
    \fn Kalcoolus::slotNumberButton()
 */
void Kalcoolus::slotNumberButton()
{
  /* sender()->name() tells us which button was pressed
   * then we map it into a operand symbol */
  QString digit = m_buttonsMap[sender()->name()];
  insertStringIntoDisplay(digit, 0, 1);
  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotOperatorButton()
 */
void Kalcoolus::slotOperatorButton()
{
  /* sender()->name() tells us which button was pressed
   * then we map it into a operand symbol */
  QString operation = m_buttonsMap[sender()->name()];
  insertStringIntoDisplay(operation, 0, operation.length());
  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotFunctionButton()
 */
void Kalcoolus::slotFunctionButton()
{
  /* sender()->name() tells us which button was pressed
     then we map it into a function symbol */
  QString function = m_buttonsMap[sender()->name()];
  insertStringIntoDisplay(function, 0, function.length()-1);
  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotImaginaryButton()
 */
void Kalcoolus::slotImaginaryButton()
{
 /* sender()->name() tells us which button was pressed
  * then we map it into a operand symbol */
  QString im = m_buttonsMap[sender()->name()];
  insertStringIntoDisplay(im, 0, 1);
  m_inputBuffer = m_inputDisplay->text(); 
}

/*!
    \fn Kalcoolus::slotResultButton()
 */
void Kalcoolus::slotLastResultButton()
{
  insertStringIntoDisplay(m_last, 0, m_last.length());
  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotEraseClicked()
 */
void Kalcoolus::slotEraseClicked()
{
  // Cleaning all 
  m_inputDisplay->clear();
  m_inputDisplay->setText("\n");
  m_resultDisplay->clear();
  m_inputBuffer = m_inputDisplay->text();
  m_inputDisplay->setCursorPosition(0,0);
  m_resultDisplay->setCursorPosition(0,0);
}

/*!
    \fn Kalcoolus::slotCancelClicked()
 */
void Kalcoolus::slotCancelClicked()
{
  // We want to cancel a character from the cursor position 
  int para,
      index;
  m_inputDisplay->getCursorPosition(&para,&index);
  // Splitting text into single lines 
  QStringList inputDisplayList = QStringList::split("\n", m_inputDisplay->text(), true);
  /* Deleting characters. */
  // Updating cursor position
  index --;
  if(index < 0)
    {
      para --;
      if(para < 0)
        {
          para = 0;
          index = 0;
        }
      else
        index = inputDisplayList[para].length()-1;
    }
  if(index == 0 and inputDisplayList[para][index+1] == '\n')
    inputDisplayList[para].remove(index, 2);
  else
    inputDisplayList[para].remove(index, 1);
  // Reconstructing text 
  m_inputDisplay->setText(inputDisplayList.join("\n"));
  m_inputDisplay->setCursorPosition(para, index);

  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotCopyrights()
 */
void Kalcoolus::slotCopyrights()
{
  KProcess* proc = new KProcess();

  proc->setUseShell(true);

  QString mode;

  if(not m_bComplex->isOn())
    {
      mode = "bc";
      *proc << m_configuration->bcPath() <<  KProcess::quote("-v");
    }
  else
    {
      mode = "python";
      *proc << "python -c \"copyright()\"";
    }

  connect(proc, SIGNAL(receivedStdout(KProcess*,char*,int)), this, SLOT(slotGetScriptOutput(KProcess*,char*,int)));
  connect(proc, SIGNAL(receivedStderr(KProcess*,char*,int)), this, SLOT(slotGetScriptError(KProcess*,char*,int)));
  connect(proc, SIGNAL(processExited(KProcess*)), this, SLOT(slotProcessExited(KProcess*)));

  if(not proc->start(KProcess::Block, KProcess::All))
    KMessageBox::error(0,"Error: cannot retrive "+mode+" copyrights notice");

  if(proc) delete proc;
}

/*!
    \fn Kalcoolus::slotNewLineClicked()
 */
void Kalcoolus::slotNewLineClicked()
{
  // Going to the next line 
  int para,
      index;
  m_inputDisplay->getCursorPosition(&para,&index);
  // Advancing of one paragraph 
  para++;
  // index must be at the beginning of the line 
  index = 0;
  m_inputDisplay->insertAt("\n", para, 0);
  // Reset cursor postion 
  m_inputDisplay->setCursorPosition(para, 0);

  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotEqualClicked()
 */
void Kalcoolus::slotEqualClicked()
{
  if(m_inputDisplay->text() == "\n") return;
  // Cleaning display 
  m_resultDisplay->clear(); 
  // Creating the internal bc/python script
  CodeGenerator codeGenerator(m_inputDisplay->text(),
                              m_languagesMap,
                              m_constantsMap,
                              m_bcScriptFilePath,
                              m_pythonScriptFilePath,
                              m_configuration);
  QString result;
  if(m_bComplex->isOn())
    codeGenerator.executeProgramPython(result);
  else
    codeGenerator.executeProgramBC(result);

  QStringList resultList = QStringList::split("\n",result);
  m_last = resultList[resultList.count()-1];

  m_resultDisplay->setText(result);

  // Enabling log file features
  if(m_configuration->enableLogFile())
    {
      QString logInfo = CodeGenerator::Mode + " - " + QDate::currentDate(Qt::LocalTime).toString(Qt::LocalDate)+"\n";
      QFile logFile(m_configuration->logFilePath());
      if(logFile.open( IO_WriteOnly | IO_Append ))
        {
          QTextStream stream( &logFile );
          stream << logInfo << m_inputDisplay->text();
        }
      logFile.close();
    }
}

/*!
    \fn Kalcoolus::slotAddUserFuntion()
 */
void Kalcoolus::slotAddUserFunction()
{
  QString userFunctionName = m_leUserFunctionName->text(),
          userFunctionArguments = m_leUserFunctionArguments->text(),
          userFunctionBody = m_editorUserFunctionBody->text();

  userFunctionName.remove(" ");
  userFunctionArguments.remove(" ");

  if(userFunctionName.isEmpty() or (userFunctionName.contains(QRegExp("^[A-Z]\\w*")) != 0))
    {
      KMessageBox::error(0,"Function name must be not empty and must begin with a lower case letter");
      return;
    }

  if(m_bComplex->isOn())
    {
      QStringList instructionList = QStringList::split("\n",userFunctionBody);
      uint j = 0,
           count = instructionList.count();
      QString body;
      while(j < count)
        {
          PythonCodeGenerator::convertToPython(instructionList[j],
                                               m_languagesMap["python"],
                                               m_constantsMap);
          if(instructionList[j][0] == ' ')
            body += instructionList[j] + "\n";
          else
            body += " " + instructionList[j] + "\n";
          j++;
        }
      QString userFunctionCode = QString("def %1 (%2):\n%3\n\n").
                                     arg(userFunctionName).
                                     arg(userFunctionArguments).
                                     arg(body);
      userFunctionCode.stripWhiteSpace();
      m_languagesMap["python"][userFunctionName] = userFunctionCode;
    }
  else
    {
      QStringList instructionList = QStringList::split("\n",userFunctionBody);
      uint j = 0,
           count = instructionList.count();
      QString body;
      while(j < count)
        {
          BCCodeGenerator::convertToBC(instructionList[j],
                                       m_languagesMap["bc"],
                                       m_constantsMap);
          if(instructionList[j][0] == ' ') 
            body += instructionList[j]+"\n";
          else
            body += " "+instructionList[j]+"\n";
          j++;
        }
      QString userFunctionCode = QString("define %1 (%2) {\n%3\n}\n").
                                     arg(userFunctionName).
                                     arg(userFunctionArguments).
                                     arg(body);
      userFunctionCode.stripWhiteSpace();
      m_languagesMap["bc"][userFunctionName] = userFunctionCode;
    }

  if(m_cbxUserFunctionsList->currentText() != userFunctionName)
    m_cbxUserFunctionsList->insertItem(userFunctionName);

  m_cbxUserFunctionsList->setCurrentText(userFunctionName);

  if(m_cbxFunctionsBox->currentText() != userFunctionName)
    m_cbxFunctionsBox->insertItem(userFunctionName);

  m_cbxFunctionsBox->setCurrentText(userFunctionName);

  m_leUserFunctionName->clear();
  m_leUserFunctionArguments->clear();
  m_editorUserFunctionBody->clear();
}

/*!
    \fn Kalcoolus::slotDeleteUserFuntion()
 */
void Kalcoolus::slotDeleteUserFunction()
{
  QString currentFunctionName = m_cbxUserFunctionsList->currentText();

  if(m_bComplex->isOn())
    m_languagesMap["python"].remove(currentFunctionName);
  else
    m_languagesMap["bc"].remove(currentFunctionName);

  m_leUserFunctionName->clear();
  m_leUserFunctionArguments->clear();
  m_editorUserFunctionBody->clear();

  m_cbxUserFunctionsList->removeItem(m_cbxUserFunctionsList->currentItem());

  int i = 0;
  while(i < m_cbxFunctionsBox->count())
    {
      if(currentFunctionName == m_cbxFunctionsBox->text(i))
        {
          m_cbxFunctionsBox->removeItem(i);
          break;
        }
      else i++;
    }
}

/*!
    \fn Kalcoolus::slotEditUserFuntion(int index)
 */
void Kalcoolus::slotEditUserFunction(int index)
{
  Q_UNUSED(index);
  if(m_bComplex->isOn())
    {
      QString currentText = m_cbxUserFunctionsList->currentText();
      QRegExp r = QRegExp(pythonFunctionNamePattern);
              r.search(m_languagesMap["python"][currentText]);
              m_leUserFunctionName->setText(r.cap(1));

              r = QRegExp(pythonFunctionArgumentsPattern);
              r.search(m_languagesMap["python"][currentText]);
              m_leUserFunctionArguments->setText(r.cap(1));

              r = QRegExp(pythonFunctionBodyPattern);
              r.search(m_languagesMap["python"][currentText]);
      QString functionBody = r.cap(1);
              PythonCodeGenerator::convertToKalcoolus(functionBody);

      int k = functionBody.length()-1;
              while(QString("\n ").contains(functionBody[k]) != 0) 
                {
                   if(k >= 0) functionBody.truncate(k);
                   k--;
                }
      m_editorUserFunctionBody->setText(functionBody);
      if(functionBody[0] == '\t') functionBody.remove(0,1);
    }
  else
    {
      QString currentText = m_cbxUserFunctionsList->currentText();
      QRegExp r = QRegExp(bcFunctionNamePattern);
              r.search(m_languagesMap["bc"][currentText]);
              m_leUserFunctionName->setText(r.cap(1));

              r = QRegExp(bcFunctionArgumentsPattern);
              r.search(m_languagesMap["bc"][currentText]);
              m_leUserFunctionArguments->setText(r.cap(1));

              r = QRegExp(bcFunctionBodyPattern);
              r.search(m_languagesMap["bc"][currentText]);
      QString functionBody = r.cap(1);
              BCCodeGenerator::convertToKalcoolus(functionBody);
              m_editorUserFunctionBody->setText(functionBody.stripWhiteSpace());
    }
}

/*!
    \fn Kalcoolus::slotSelectUserFunction(const QString& s)
 */
void Kalcoolus::slotSelectUserFunction(const QString& s)
{
  QString currentItem = s,
          language,
          patternName,
          patternArgs,
          patternBody;
  if(m_bComplex->isOn())
    {
      language = "python";
      patternName = pythonFunctionNamePattern;
      patternArgs = pythonFunctionArgumentsPattern;
      patternBody = pythonFunctionBodyPattern;
    }
  else
    {
      language = "bc";
      patternName = bcFunctionNamePattern;
      patternArgs = bcFunctionArgumentsPattern;
      patternBody = bcFunctionBodyPattern;
    }
  QRegExp r = QRegExp(patternName);
          r.search(m_languagesMap[language][currentItem]);
  QString function = r.cap(1);

          r = QRegExp(patternArgs);
          r.search(m_languagesMap[language][currentItem]);
  QString argList = r.cap(1);

  if(argList.isEmpty()) function += "(#)";
  else
    {
      int i = argList.contains(",");
      if(i != 0) function += "(" + QString().fill(',',i)+")";
      else function +="()";
    }

  insertStringIntoDisplay(function, 0, function.length() - argList.contains(",") - 1);

  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotConstantsBox(const QString &)
 */
void Kalcoolus::slotConstantsBox(const QString & s)
{
  insertStringIntoDisplay(s, 0, s.length());
  m_inputBuffer = m_inputDisplay->text();
}

/*!
    \fn Kalcoolus::slotBye()
 */
void Kalcoolus::slotBye()
{
  accept();
}

/*!
    \fn Kalcoolus::slotSave()
 */
void Kalcoolus::slotSave()
{
  // Saving BC settings 
  writeBCSettings();
  // Saving Python settings 
  writePythonSettings();
  // Saving font settings
  writeFontSettings();
  // Saving option settings
  writeOptionsSettings();
  // Saving constants/functions code to databases 
  m_constantsDB->writeToDatabase();
  m_functionsDB->writeToDatabase(m_languagesMap);
}

/*!
    \fn Kalcoolus::slotComplexMode()
 */
void Kalcoolus::slotComplexMode()
{
  /** Enabling/disabling capabilities when we switch between 
   *  normal and complex mode */
  loadFunctionsIntoBoxes();
  bool complexMode = m_bComplex->isOn();

  m_bImaginary->setEnabled(complexMode);
  m_bLess->setEnabled(not complexMode);
  m_bLessEqual->setEnabled(not complexMode);
  m_bGreat->setEnabled(not complexMode);
  m_bGreatEqual->setEnabled(not complexMode);
  m_bPower->setEnabled(not complexMode);
  m_bPercentage->setEnabled(not complexMode);
  m_bRealPart->setEnabled(complexMode);
  m_bImaginaryPart->setEnabled(complexMode);
  m_bConjugate->setEnabled(complexMode);
  m_bArgument->setEnabled(complexMode);
  m_bModulus->setEnabled(complexMode);
  m_configView->findItem("Setting python",0)->setSelectable(complexMode);
  m_configView->findItem("Setting bc",0)->setSelectable(not complexMode);

  if(m_bComplex->isOn())
    {
      m_configTool->raiseWidget(2);
      m_configTitle->setText("General python language settings");
      m_grpboxExpressions->setTitle("Expressions : complex mode");
      m_grpboxResults->setTitle("Results : complex mode");
    }
  else
    {
      m_configTool->raiseWidget(0);
      m_configTitle->setText("General bc language settings");
      m_configView->firstItem()->setSelectable(true);
      m_grpboxExpressions->setTitle("Expressions : normal mode");
      m_grpboxResults->setTitle("Results : normal mode");
    }
}

// Constants Page Slots 
/*!
    \fn Kalcoolus::slotAddConstant()
 */
void Kalcoolus::slotAddConstant()
{
  QString name = m_leConstName->text(),
          value = m_leConstValue->text();
  if(not name.isEmpty() and not value.isEmpty())
    {
      QListViewItem* item = new KListViewItem(m_constantsListview);
      item->setText(0, name);
      item->setText(1, value);
      m_constantsMap[name] = value;
      m_leConstName->clear();
      m_leConstValue->clear();
    }
  loadConstantsIntoBox();
}

/*!
    \fn Kalcoolus::slotClearConstant()
 */
void Kalcoolus::slotClearConstant()
{
  int i;
  for (i = 0; i < m_cbxConstantsBox->count(); i++)
    {
      if(m_cbxConstantsBox->text(i) == m_leConstName->text())
        {
          m_cbxConstantsBox->removeItem(i);
          break;
        }
    }
  m_leConstName->clear();
  m_leConstValue->clear();
}

/*!
    \fn Kalcoolus::slotConstSelected(QListViewItem*)
 */
void Kalcoolus::slotConstSelected(QListViewItem* item)
{
  /* When a constant is selected in the constants page it is removed from
     memory until we have finished to reedit its value */
  if(item)
    {
      m_leConstName->setText(item->text(0));
      m_leConstValue->setText(item->text(1));
      m_constantsMap.remove(item->text(0));
      delete item;
    }
}
// Configuration Page Slots
/*!
    \fn Kalcoolus::slotScale(const QString& s)
 */
void Kalcoolus::slotScale(const QString& s)
{
  m_configuration->setScale(s);
}

/*!
    \fn Kalcoolus::slotIBase(const QString& s)
 */
void Kalcoolus::slotIBase(const QString& s)
{
  m_configuration->setIBase(s);
  activateButtons();
}

/*!
    \fn Kalcoolus::slotOBase(const QString& s)
 */
void Kalcoolus::slotOBase(const QString& s)
{
  m_configuration->setOBase(s);
}

/*!
    \fn Kalcoolus::slotPyDecimalDigits(const QString& s)
 */
void Kalcoolus::slotPyDecimalDigits(const QString& s)
{
  m_configuration->setPythonDecimalDigits(s);
}

/*!
    \fn Kalcoolus::slotChooseConfigItem(QIconViewItem*)
 */
void Kalcoolus::slotChooseConfigItem(QIconViewItem* item)
{
  // Enabling choosed configuration item 
  if(not item) return;

  QString itemCaption = item->text();
  if(itemCaption == "Setting bc" and not m_bComplex->isOn())
    {
      m_configTool->raiseWidget(0);
      m_configTitle->setText("General bc language settings");
    }
  else
  if(itemCaption == "Fonts")
    {
      m_configTool->raiseWidget(1);
      m_configTitle->setText("Display font");
    }
  else
  if(itemCaption == "Setting python" and m_bComplex->isOn())
    {
      m_configTool->raiseWidget(2);
      m_configTitle->setText("General python language settings");
    }
  else
  if(itemCaption == "Options")
    {
      m_configTool->raiseWidget(3);
      m_configTitle->setText("Options settings");
    }
}

/*!
    \fn Kalcoolus::slotFontSelected(const QFont& f)
 */
void Kalcoolus::slotFontSelected(const QFont& f)
{
  m_inputDisplay->setFont(f);
  m_resultDisplay->setFont(f);
  m_editorUserFunctionBody->setFont(f);
}

/*!
    \fn Kalcoolus::slotColorNumbers(const QColor& c)
 */
void Kalcoolus::slotColorNumbers(const QColor& c)
{
  m_configuration->setColor(c,numbers);
  m_inputHighlighter->setColor(c,numbers);
  m_resultHighlighter->setColor(c,numbers);
  m_functionEditorHighlighter->setColor(c,numbers);
  setNumberButtonsColor(c);
}

/*!
    \fn Kalcoolus::slotColorOperations(const QColor& c)
 */
void Kalcoolus::slotColorOperations(const QColor& c)
{
  m_configuration->setColor(c,operations);
  m_inputHighlighter->setColor(c,operations);
  m_resultHighlighter->setColor(c,operations);
  m_functionEditorHighlighter->setColor(c,operations);
  setOperationButtonsColor(c);
}

/*!
    \fn Kalcoolus::slotColorFunctions(const QColor& c)
 */
void Kalcoolus::slotColorFunctions(const QColor& c)
{
  m_configuration->setColor(c,functions);
  m_inputHighlighter->setColor(c,functions);
  m_resultHighlighter->setColor(c,functions);
  m_functionEditorHighlighter->setColor(c,functions);
  setFunctionButtonsColor(c);
}

/*!
    \fn Kalcoolus::slotColorKeywords(const QColor& c)
 */
void Kalcoolus::slotColorKeywords(const QColor& c)
{
  m_configuration->setColor(c,keywords);
  m_inputHighlighter->setColor(c,keywords);
  m_resultHighlighter->setColor(c,keywords);
  m_functionEditorHighlighter->setColor(c,keywords);
  setKeywordButtonsColor(c);
}

/*!
    \fn Kalcoolus::slotColorSpecial(const QColor& c)
 */
void Kalcoolus::slotColorSpecial(const QColor& c)
{
  m_configuration->setColor(c,specials);
  m_inputHighlighter->setColor(c,specials);
  m_resultHighlighter->setColor(c,specials);
  m_functionEditorHighlighter->setColor(c,specials);
  setSpecialButtonsColor(c);
}

/*!
    \fn Kalcoolus::slotEnableLogFile(bool b)
*/
void Kalcoolus::slotEnableLogFile(bool b)
{
  m_kurlLogFilePath->setEnabled(b);
}

// Other Slots 
/*!
    \fn Kalcoolus::slotHelp()
 */
void Kalcoolus::slotHelp()
{
  kapp->invokeHelp("","","");
}

/*******************************************************************/
/************************ PRIVATE METHODS **************************/
/*******************************************************************/

/*!
    \fn Kalcoolus::Connect()
 */
void Kalcoolus::ConnectKeyboardPage()
{
  // Connecting Keyboard Page signals 
  connect(m_b1, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b2, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b3, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b4, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b5, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b6, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b7, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b8, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b9, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_b0, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_bA, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_bB, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_bC, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_bD, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_bE, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_bF, SIGNAL(clicked()), this, SLOT(slotNumberButton()));
  connect(m_bImaginary, SIGNAL(clicked()), this, SLOT(slotImaginaryButton()));

  connect(m_bDelete, SIGNAL(clicked()), this, SLOT(slotEraseClicked()));
  connect(m_bCancel, SIGNAL(clicked()), this, SLOT(slotCancelClicked()));
  connect(m_bNewLine, SIGNAL(clicked()), this, SLOT(slotNewLineClicked()));
  connect(m_bLastResult, SIGNAL(clicked()), this, SLOT(slotLastResultButton()));

  connect(m_bDiv, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bDot, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bMult, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bPlus, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bMinus, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bEqual, SIGNAL(clicked()), this, SLOT(slotEqualClicked()));

  connect(m_bGreat, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bLess, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bGreatEqual, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bLessEqual, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bIdentical, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bDifferent, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bLeftRound, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));
  connect(m_bRightRound, SIGNAL(clicked()), this, SLOT(slotOperatorButton()));

  connect(m_bSin, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bCos, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bTan, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bAtan, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bLog, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bLn, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bExp, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bSqr, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bSqrt, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bPower, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bFact, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bPercentage, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bWarning, SIGNAL(clicked()), this, SLOT(slotCopyrights()));
  connect(m_bRealPart, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bImaginaryPart, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bConjugate, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bArgument, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_bModulus, SIGNAL(clicked()), this, SLOT(slotFunctionButton()));
  connect(m_cbxFunctionsBox, SIGNAL(activated(const QString &)), this, SLOT(slotSelectUserFunction(const QString&)));
  connect(m_cbxConstantsBox, SIGNAL(activated(const QString &)), this, SLOT(slotConstantsBox(const QString &)));

  connect(m_bBye, SIGNAL(clicked()), this, SLOT(slotBye()));
  connect(m_bSave, SIGNAL(clicked()), this, SLOT(slotSave()));

  connect(m_bComplex, SIGNAL(clicked()), this, SLOT(slotComplexMode()));
}

/*!
    \fn Kalcoolus::ConnectConstantsPage()
 */
void Kalcoolus::ConnectConstantsPage()
{
  // Connecting Constants Page signals 
  connect(m_bConstAdd, SIGNAL(clicked()), this, SLOT(slotAddConstant()));
  connect(m_bConstClear, SIGNAL(clicked()), this, SLOT(slotClearConstant()));
  connect(m_constantsListview, SIGNAL(clicked(QListViewItem*)), this, SLOT(slotConstSelected(QListViewItem*)));
}

/*!
    \fn Kalcoolus::ConnectFunctionsPage
 */
void Kalcoolus::ConnectFunctionsPage()
{
  // Connecting Functions Page signals 
  connect(m_bAddUserFunction, SIGNAL(clicked()), this, SLOT(slotAddUserFunction()));
  connect(m_bDeleteUserFunction, SIGNAL(clicked()), this, SLOT(slotDeleteUserFunction()));
  connect(m_cbxUserFunctionsList, SIGNAL(activated(int)), this, SLOT(slotEditUserFunction(int)));
}

/*!
    \fn Kalcoolus::ConnectConfigurationPage
 */
void Kalcoolus::ConnectConfigurationPage()
{
  // Connecting Configuration Page signals 
  connect(m_configView, SIGNAL(clicked(QIconViewItem*)), this, SLOT(slotChooseConfigItem(QIconViewItem*)));
  connect(m_spbxScale, SIGNAL(valueChanged(const QString&)), this, SLOT(slotScale(const QString&)));
  connect(m_spbxIBase, SIGNAL(valueChanged(const QString&)), this, SLOT(slotIBase(const QString&)));
  connect(m_spbxOBase, SIGNAL(valueChanged(const QString&)), this, SLOT(slotOBase(const QString&)));
  connect(m_spbxPyDecimalDigits, SIGNAL(valueChanged(const QString&)), this, SLOT(slotPyDecimalDigits(const QString&)));
  connect(m_fontRequester, SIGNAL(fontSelected(const QFont&)), this, SLOT(slotFontSelected(const QFont&)));
  connect(m_colbxNumbers, SIGNAL(activated(const QColor&)), this, SLOT(slotColorNumbers(const QColor&)));
  connect(m_colbxOperations, SIGNAL(activated(const QColor&)), this, SLOT(slotColorOperations(const QColor&)));
  connect(m_colbxFunctions, SIGNAL(activated(const QColor&)), this, SLOT(slotColorFunctions(const QColor&)));
  connect(m_colbxKeywords, SIGNAL(activated(const QColor&)), this, SLOT(slotColorKeywords(const QColor&)));
  connect(m_colbxSpecials, SIGNAL(activated(const QColor&)), this, SLOT(slotColorSpecial(const QColor&)));
  connect(m_cbxEnableLogFile, SIGNAL(toggled(bool)), this, SLOT(slotEnableLogFile(bool)));
}

/*!
    \fn Kalcoolus::mapButtons()
 */
void Kalcoolus::mapButtons()
{
  /** This object maps the names of the GUI buttons
   *  to mathematical symbols */
  m_buttonsMap["m_b0"] = "0";
  m_buttonsMap["m_b1"] = "1";
  m_buttonsMap["m_b2"] = "2";
  m_buttonsMap["m_b3"] = "3";
  m_buttonsMap["m_b4"] = "4";
  m_buttonsMap["m_b5"] = "5";
  m_buttonsMap["m_b6"] = "6";
  m_buttonsMap["m_b7"] = "7";
  m_buttonsMap["m_b8"] = "8";
  m_buttonsMap["m_b9"] = "9";
  m_buttonsMap["m_bA"] = "A";
  m_buttonsMap["m_bB"] = "B";
  m_buttonsMap["m_bC"] = "C";
  m_buttonsMap["m_bD"] = "D";
  m_buttonsMap["m_bE"] = "E";
  m_buttonsMap["m_bF"] = "F";

  m_buttonsMap["m_bImaginary"] = "i";

  m_buttonsMap["m_bPlus"] = "+";
  m_buttonsMap["m_bMinus"] = "-";
  m_buttonsMap["m_bMult"] = "*";
  m_buttonsMap["m_bDiv"] = "/";
  m_buttonsMap["m_bLess"] = "<";
  m_buttonsMap["m_bGreat"] = ">";
  m_buttonsMap["m_bLessEqual"] = "<=";
  m_buttonsMap["m_bGreatEqual"] = ">=";
  m_buttonsMap["m_bIdentical"] = "==";
  m_buttonsMap["m_bDifferent"] = "!=";
  m_buttonsMap["m_bDot"] = ".";
  m_buttonsMap["m_bPercent"] = "%";
  m_buttonsMap["m_bLeftRound"] = "(";
  m_buttonsMap["m_bRightRound"] = ")";
  m_buttonsMap["m_bLeftSquare"] = "[";
  m_buttonsMap["m_bRightSquare"] = "]";

  m_buttonsMap["m_bSin"] = "sin()";
  m_buttonsMap["m_bCos"] = "cos()";
  m_buttonsMap["m_bTan"] = "tan()";
  m_buttonsMap["m_bAtan"] = "atan()";
  m_buttonsMap["m_bLog"] = "log()";
  m_buttonsMap["m_bLn"] = "ln()";
  m_buttonsMap["m_bExp"] = "exp()";
  m_buttonsMap["m_bSqr"] = "sqr()";
  m_buttonsMap["m_bSqrt"] = "sqrt()";
  m_buttonsMap["m_bPower"] = "pow(,)";
  m_buttonsMap["m_bFact"] = "fact()";
  m_buttonsMap["m_bPercentage"] = "percent(,)";
  m_buttonsMap["m_bWarning"] = "copyrights(#)";
  m_buttonsMap["m_bRealPart"] = "re()";
  m_buttonsMap["m_bImaginaryPart"] = "im()";
  m_buttonsMap["m_bConjugate"] = "conj()";
  m_buttonsMap["m_bArgument"] = "arg()";
  m_buttonsMap["m_bModulus"] = "mod()";
}

/*!
    \fn Kalcoolus::loadConstantsIntoBox()
 */
void Kalcoolus::loadConstantsIntoBox()
{
  // Adding constants to the user-defined list 
  m_cbxConstantsBox->clear();
  QListViewItem* tempItem = m_constantsListview->firstChild();
  while(tempItem)
    {
      m_cbxConstantsBox->insertItem(tempItem->text(0));
      m_constantsMap[tempItem->text(0)] = tempItem->text(1);
      tempItem = tempItem->nextSibling();
    }
}

/*!
    \fn Kalcoolus::loadFunctionsIntoBoxes()
 */
void Kalcoolus::loadFunctionsIntoBoxes()
{
  // Adding user-defined function name to the list 
  m_cbxUserFunctionsList->clear();
  m_cbxFunctionsBox->clear();

  FunctionsCodeMap fcMap;
  QRegExp r;
  if(m_bComplex->isOn())
    {
      r = QRegExp(pythonFunctionNamePattern);
      fcMap = m_languagesMap["python"];
    }
  else
    {
      r = QRegExp(bcFunctionNamePattern);
      fcMap = m_languagesMap["bc"];
    }
  FunctionsCodeMap::Iterator it;
  for(it = fcMap.begin(); it != fcMap.end(); ++it)
    {
      r.search((*it).data());
      QString functionName = r.cap(1);
      m_cbxUserFunctionsList->insertItem(functionName);
      m_cbxFunctionsBox->insertItem(functionName);
    }
}

/*!
    \fn Kalcoolus::loadFontSettings()
 */
void Kalcoolus::loadFontSettings()
{
  // Loading from kalcoolusrc file to configuration object 
  m_config->setGroup("Font Settings");
  m_configuration->setFont(m_config->readEntry("Font",FontValue));

  // Loading values from configuration object to GUI objects 
  QFont f;
  f.fromString(m_configuration->font());

  // Setting font of the edit areas 
  m_fontRequester->setFont(f);
  m_inputDisplay->setFont(f);
  m_resultDisplay->setFont(f);
  m_editorUserFunctionBody->setFont(f);

  // Setting numbers color
  QString color = m_config->readEntry("Numbers color", NumbersColor);
          setGUIColor(color, numbers);

  // Setting number operation/operator colors 
  color = m_config->readEntry("Operations color", OperationsColor);
  setGUIColor(color, operations);

  // Setting function name colors 
  color = m_config->readEntry("Functions color", FunctionsColor);
  setGUIColor(color, functions);

  // Setting keyword colors
  color = m_config->readEntry("Keywords color", KeywordsColor); 
  setGUIColor(color, keywords);

  // Setting special colors
  color = m_config->readEntry("Specials color", SpecialsColor); 
  setGUIColor(color, specials);
}

/*!
    \fn Kalcoolus::writeFontSettings()
 */
void Kalcoolus::writeFontSettings()
{
  QString spinBoxValue;
  // Writing GUI objects data to configuration object 
  m_configuration->setFont(m_fontRequester->font().toString());
  // Writing configuration objects data to kalcoolusrc file 
  m_config->setGroup("Font Settings");
  QString font = m_configuration->font();
  m_config->writeEntry("Font", font);

  m_config->writeEntry("Numbers color", m_configuration->color(numbers));
  m_config->writeEntry("Operations color", m_configuration->color(operations));
  m_config->writeEntry("Functions color", m_configuration->color(functions));
  m_config->writeEntry("Keywords color", m_configuration->color(keywords));
  m_config->writeEntry("Specials color", m_configuration->color(specials));
  m_config->sync();
}

/*!
    \fn Kalcoolus::loadBCSettings()
 */
void Kalcoolus::loadBCSettings()
{
  // Loading from kalcoolusrc file to configuration object 
  m_config->setGroup("BC Settings");
  m_configuration->setScale(m_config->readEntry("Scale",ScaleValue));
  m_configuration->setIBase(m_config->readEntry("IBase",IBaseValue));
  m_configuration->setOBase(m_config->readEntry("OBase",OBaseValue));
  m_configuration->setBCPath(m_config->readEntry("BC Path",BCPathValue));
  // Loading from configuration object to GUI objects 
  m_spbxScale->setValue(m_configuration->scale().toInt());
  m_spbxIBase->setValue(m_configuration->ibase().toInt());
  m_spbxOBase->setValue(m_configuration->obase().toInt());
  m_kurlBCPath->setKURL(m_configuration->bcPath());
}

/*!
    \fn Kalcoolus::writeBCSettings()
 */
void Kalcoolus::writeBCSettings()
{
  QString spinBoxValue;
  // Writing GUI objects data to configuration object 
  m_configuration->setScale(spinBoxValue.setNum(m_spbxScale->value()));
  m_configuration->setIBase(spinBoxValue.setNum(m_spbxIBase->value()));
  m_configuration->setOBase(spinBoxValue.setNum(m_spbxOBase->value()));
  m_configuration->setBCPath(m_kurlBCPath->url());
  // Writing configuration objects data to kalcoolusrc file 
  m_config->setGroup("BC Settings");
  m_config->writeEntry("Scale", m_configuration->scale());
  m_config->writeEntry("IBase", m_configuration->ibase());
  m_config->writeEntry("OBase", m_configuration->obase());
  m_config->writeEntry("BC Path", m_configuration->bcPath());
  m_config->sync();
}

/*!
    \fn Kalcoolus::loadPythonSettings()
 */
void Kalcoolus::loadPythonSettings()
{
  // Loading from kalcoolusrc file to configuration object 
  m_config->setGroup("Python Settings");
  m_configuration->setPythonDecimalDigits(m_config->readEntry("Python Decimal Digits",PythonDecimalDigitsValue));
  m_configuration->setPythonModulesPath(m_config->readEntry("Python Modules Path",PythonModulesPathValue));
  // Loading from configuration object to GUI objects 
  m_kurlPythonModulesPath->setKURL(m_configuration->pythonModulesPath());
  m_spbxPyDecimalDigits->setValue(m_configuration->pythonDecimalDigits().toInt());
}

/*!
    \fn Kalcoolus::writePythonSettings()
 */
void Kalcoolus::writePythonSettings()
{
  QString spinBoxValue;
  // Writing GUI objects data to configuration object 
  m_configuration->setPythonDecimalDigits(spinBoxValue.setNum(m_spbxPyDecimalDigits->value()));
  m_configuration->setPythonModulesPath(m_kurlPythonModulesPath->url());
  // Writing configuration objects data to kalcoolusrc file 
  m_config->setGroup("Python Settings");
  m_config->writeEntry("Python Decimal Digits", m_configuration->pythonDecimalDigits());
  m_config->writeEntry("Python Modules Path", m_configuration->pythonModulesPath());
  m_config->sync();
}

/*!
    \fn Kalcoolus::loadOptionsSettings()
 */
void Kalcoolus::loadOptionsSettings()
{
  // Loading from kalcoolusrc file to configuration object 
  m_config->setGroup("Options Settings");
  m_configuration->setEnableLogFile(m_config->readBoolEntry("Enable Log File", false));
  if(m_configuration->enableLogFile())
    m_configuration->setLogFilePath(m_config->readEntry("Log File Path",LogFilePathValue));
  // Loading from configuration object to GUI objects 
  m_cbxEnableLogFile->setChecked(m_configuration->enableLogFile());
  m_kurlLogFilePath->setKURL(m_configuration->logFilePath());
  m_kurlLogFilePath->setEnabled(m_configuration->enableLogFile());
}

/*!
    \fn Kalcoolus::writeOptionsSettings()
 */
void Kalcoolus::writeOptionsSettings()
{
  // Writing GUI objects data to configuration object 
  m_configuration->setEnableLogFile(m_cbxEnableLogFile->isChecked());
  m_configuration->setLogFilePath(m_kurlLogFilePath->url());
  // Writing configuration objects data to kalcoolusrc file 
  m_config->setGroup("Options Settings");
  m_config->writeEntry("Enable Log File", m_configuration->enableLogFile());
  if(m_configuration->enableLogFile())
    m_config->writeEntry("Log File Path", m_configuration->logFilePath());

  m_config->sync();
}

/*!
    \fn Kalcoolus::activateButtons()
 */
void Kalcoolus::activateButtons()
{
  // Enabling hexadecimal buttons when numerical base changes 
  QMap<int, KPushButton*> numButton;

  numButton[2] = m_b1;
  numButton[3] = m_b2;
  numButton[4] = m_b3;
  numButton[5] = m_b4;
  numButton[6] = m_b5;
  numButton[7] = m_b6;
  numButton[8] = m_b7;
  numButton[9] = m_b8;
  numButton[10] = m_b9;
  numButton[11] = m_bA;
  numButton[12] = m_bB;
  numButton[13] = m_bC;
  numButton[14] = m_bD;
  numButton[15] = m_bE;
  numButton[16] = m_bF;

  int i,
      inputBaseValue = m_configuration->ibase().toInt();
  for (i = 2; i <= 16; i++)
    if(i <= inputBaseValue)
      numButton[i]->setEnabled(true);
    else
      numButton[i]->setEnabled(false);
}

/*!
    \fn Kalcoolus::createHelp()
 */
void Kalcoolus::createHelp()
{
  // Connecting help 
  KAboutData* aboutData = new KAboutData("Kalcoolus",
                                         I18N_NOOP("Kalcoolus"),
                                         "0.1.8",
                                         I18N_NOOP("This is a KDE calculator built on top of GNU bc 1.06, with support for python 2.3.4"),
                                         KAboutData::License_GPL_V2,
                                         "(c) 2005, Emiliano Gulmini",
                                         I18N_NOOP("\n"),
                                         "http://web.tiscali.it/gulbros","submit@bugs.kde.org");

  aboutData->addAuthor("Emiliano Gulmini",
                       I18N_NOOP("Original author and current maintainer"),
                       "emi_barbarossa@yahoo.it",
                       "http://web.tiscali.it/gulbros");

  aboutData->setOtherText("Articolo 11 -  L'Italia ripudia la guerra come\nstrumento di offesa alla liberta' degli altri popoli e come mezzo di\nrisoluzione delle controversie internazionali;\nconsente, in condizioni di parita' con gli altri Stati,\nalle limitazioni di sovranita' necessarie ad\nun ordinamento che assicuri la pace e la giustizia fra le Nazioni;\npromuove e favorisce le organizzazioni internazionali rivolte a tale scopo."); 

  connect(m_bHelp, SIGNAL(clicked()), this, SLOT(slotHelp()));

  KHelpMenu *helpMenu = new KHelpMenu(this, aboutData, true);

  m_bHelp->setPopup((QPopupMenu*)helpMenu->menu());
}

/*!
    \fn Kalcoolus::setNumberButtonsColor(const QColor& c)
 */
void Kalcoolus::setNumberButtonsColor(const QColor& c)
{
  // Setting color for number buttons 
  m_b0->setPaletteBackgroundColor(c);
  m_b1->setPaletteBackgroundColor(c);
  m_b2->setPaletteBackgroundColor(c);
  m_b3->setPaletteBackgroundColor(c);
  m_b4->setPaletteBackgroundColor(c);
  m_b5->setPaletteBackgroundColor(c);
  m_b6->setPaletteBackgroundColor(c);
  m_b7->setPaletteBackgroundColor(c);
  m_b8->setPaletteBackgroundColor(c);
  m_b9->setPaletteBackgroundColor(c);
  m_bA->setPaletteBackgroundColor(c);
  m_bB->setPaletteBackgroundColor(c);
  m_bC->setPaletteBackgroundColor(c);
  m_bD->setPaletteBackgroundColor(c);
  m_bE->setPaletteBackgroundColor(c);
  m_bF->setPaletteBackgroundColor(c);
  m_bImaginary->setPaletteBackgroundColor(c);
}

/*!
    \fn Kalcoolus::setOperationButtonsColor(const QColor& c)
 */
void Kalcoolus::setOperationButtonsColor(const QColor& c)
{
  // Setting color for operation buttons 
  m_bPlus->setPaletteBackgroundColor(c);
  m_bMult->setPaletteBackgroundColor(c);
  m_bDiv->setPaletteBackgroundColor(c);
  m_bMinus->setPaletteBackgroundColor(c);
  m_bEqual->setPaletteBackgroundColor(c);
  m_bIdentical->setPaletteBackgroundColor(c);
  m_bDifferent->setPaletteBackgroundColor(c);
  m_bGreat->setPaletteBackgroundColor(c);
  m_bGreatEqual->setPaletteBackgroundColor(c);
  m_bLess->setPaletteBackgroundColor(c);
  m_bLessEqual->setPaletteBackgroundColor(c);
  m_bComplex->setPaletteBackgroundColor(c);
  m_bLastResult->setPaletteBackgroundColor(c);
  m_bNewLine->setPaletteBackgroundColor(c);
  m_bDot->setPaletteBackgroundColor(c);
  m_bLeftRound->setPaletteBackgroundColor(c);
  m_bRightRound->setPaletteBackgroundColor(c);
}

/*!
    \fn Kalcoolus::setFunctionButtonsColor(const QColor& c)
 */
void Kalcoolus::setFunctionButtonsColor(const QColor& c)
{
  // Setting color for function buttons 
  m_bSin->setPaletteBackgroundColor(c);
  m_bCos->setPaletteBackgroundColor(c);
  m_bTan->setPaletteBackgroundColor(c);
  m_bAtan->setPaletteBackgroundColor(c);
  m_bLog->setPaletteBackgroundColor(c);
  m_bLn->setPaletteBackgroundColor(c);
  m_bExp->setPaletteBackgroundColor(c);
  m_bSqr->setPaletteBackgroundColor(c);
  m_bSqrt->setPaletteBackgroundColor(c);
  m_bPower->setPaletteBackgroundColor(c);
  m_bFact->setPaletteBackgroundColor(c);
  m_bPercentage->setPaletteBackgroundColor(c);
  m_bRealPart->setPaletteBackgroundColor(c);
  m_bImaginaryPart->setPaletteBackgroundColor(c);
  m_bConjugate->setPaletteBackgroundColor(c);
  m_bArgument->setPaletteBackgroundColor(c);
  m_bModulus->setPaletteBackgroundColor(c);
}

/*!
    \fn Kalcoolus::setKeywordButtonsColor(const QColor& c)
 */
void Kalcoolus::setKeywordButtonsColor(const QColor& c)
{Q_UNUSED(c);
}

/*!
    \fn Kalcoolus::setSpecialButtonsColor(const QColor& c)
 */
void Kalcoolus::setSpecialButtonsColor(const QColor& c)
{
  // Setting color for special buttons 
  m_bWarning->setPaletteBackgroundColor(c);
  m_bBye->setPaletteBackgroundColor(c);
  m_bSave->setPaletteBackgroundColor(c);
}

/*!
    \fn Kalcoolus::setGUIColor(QString color, Global::ColorType t)
 */
void Kalcoolus::setGUIColor(QString color, Global::ColorType t)
{

  QColor c;
  int r,g,b;
  r = color.section(",",0,0).toInt(),
  g = color.section(",",1,1).toInt(),
  b = color.section(",",2,2).toInt();
  c.setRgb(r,g,b);
  m_configuration->setColor(c,t);
  m_inputHighlighter->setColor(c,t);
  m_resultHighlighter->setColor(c,t);
  m_functionEditorHighlighter->setColor(c,t);
  switch(t)
    {
      case numbers: { m_colbxNumbers->setColor(c); 
                      setNumberButtonsColor(c); } break;

      case operations: { m_colbxOperations->setColor(c); 
                         setOperationButtonsColor(c); } break;

      case functions: { m_colbxFunctions->setColor(c); 
                        setFunctionButtonsColor(c); } break;

      case keywords: { m_colbxKeywords->setColor(c); 
                       setKeywordButtonsColor(c); } break;

      case specials: { m_colbxSpecials->setColor(c); 
                       setSpecialButtonsColor(c); } break;
    }
}

/*!
    \fn Kalcoolus::insertStringIntoDisplay(const QString& s, int dpara, int dindex)
 */
void Kalcoolus::insertStringIntoDisplay(const QString& s, int dpara, int dindex)
{
  // Getting cursor position 
  int para,
      index;
  m_inputDisplay->getCursorPosition(&para, &index);
  // Inserting the digit that maps the button 
  m_inputDisplay->insertAt(s, para, index);
  // Updating cursor position 
  m_inputDisplay->setCursorPosition(para + dpara, index+dindex);
}

void Kalcoolus::slotGetScriptError(KProcess* proc, char* s, int i)
{
  Q_UNUSED(proc);
  KMessageBox::information(0, i18n(QCString(s,i+1)), "Copyright notice");
}

void Kalcoolus::slotGetScriptOutput(KProcess* proc, char* s, int i)
{
  Q_UNUSED(proc);
  KMessageBox::information(0, i18n(QCString(s,i+1)), "Copyright notice");
}

void Kalcoolus::slotProcessExited(KProcess* proc)
{
  Q_UNUSED(proc);
}

#include "kalcoolus.moc"
